/******************************************************************************/
/* mof.c                                                                      */
/*                                                                            */
/* Contm as funes de clculo do MOF e wMOF.                                */
/******************************************************************************/
#include "funcoes.h"
#include "mof.h"


/************************************ MOF *************************************/
/* Funo que calcula o MOF de um string binrio                              */
/* Recebe:                                                                    */
/*      - um inteiro n --> o nmero de iteraes                              */
/*      - um vetor de inteiros b --> a representao binria do nmero        */
/*      - um vetor de inteiros mof, onde vai ser "devolvida" a forma MOF de d */

void MOF (int n, int b[], int mof[]) {
  int i, j;
  
  mof[n] = b[n - 1];
  for (i = n - 1; i > 0; i--) {
    mof[i] = b[i - 1] - b[i];
  }
  mof[0] = - b[0];
  
}/* MOF ----------------------------------------------------------------------*/


/******************************** calcula_MOF *********************************/
/* Funo que chama a funo que calcula o MOF de um string binrio e         */
/* imprime o resultado.                                                       */
/* Recebe:                                                                    */
/*      - um inteiro n --> o nmero de iteraes                              */
/*      - um vetor de inteiros b --> a representao binria do nmero        */

void calcula_MOF(int n, int b[]) {
  int i, mof[NMAX + 1];
  
  printf ("\n\n\n***************** MOF *****************\n\n");
  printf (" 2d = ");
  for (i = n - 1; i >= 0; i--)
    printf ("%2d ", b[i]);
  
  printf ("\n  d =    ");
  for (i = n - 1; i >= 0; i--)
    printf ("%2d ", b[i]);
  
  MOF(n, b, mof);
  
  printf ("\nMOF = ");
  for (i = n; i >=0; i--)
    printf ("%2d ", mof[i]);
  
  printf ("\n\n***************************************\n\n");
  
}/* calcula_MOF --------------------------------------------------------------*/


/*********************************** wMOF *************************************/
/* Funo que calcula o wMOF de um string binrio                             */
/* Recebe:                                                                    */
/*    - um inteiro n --> o nmero de iteraes                                */
/*    - um vetor de inteiros b --> a representao binria do nmero          */
/*    - um vetor de inteiros wmof, onde vai ser "devolvida" a forma wMOF de d */

void wMOF (int n, int w, int b[], int wmof[]) {
  int i, j, num, potencia, aux, aux1[NMAX + 1], aux2[NMAX];
  int cont = 1;

  /* Caso w == 1 */
  if (w == 1) { 
    for (i = 0; i < n; i++) { 
      wmof[i] = b[i]; 
    }
    wmof[n] = 0; 
  }

  else {
  
    /* Transformando para a forma MOF */
    MOF(n, b, aux1);
    
    i = n;
    while ((i - w) > 0) {
      num = 0;
      potencia = pot (2, w - 1);

      /* Transformando os w bits para representao decimal */
      for (j = i; j > (i - w); j--) {
	wmof[j] = 0;
	num += aux1[j] * potencia;
	potencia /= 2;
      }

      potencia = pot (2, w);
      aux = num;

      /* Preenchimento do vetor wmof: caso mpar */
      if (aux % 2 != 0) {
	if (aux < pot (2 , (w - 1))) wmof[i - w + 1] = aux;
	else wmof[i - w + 1] = aux - potencia;     
      }

      /* Preenchimento do vetor wmof: caso par */
      else {
	for(j = 0; j < w; j++) aux2[j] = 0;

	/* Preenchimento do vetor wmof: caso par e negativo */
	if (aux < 0) {
	  num = converte (-aux, aux2);
	  for (j = i, num = w - 1; j > i - w; j--) {
	    wmof[j] = - aux2[num--];
	  }
	}

	/* Preenchimento do vetor wmof: caso par e positivo */
	else {
	  num = converte (aux, aux2);
	  for (j = i, num = w - 1; j > i - w; j--) { 
	    wmof[j] = aux2[num--];
	  }
	}
      }
      cont++;

      /* Imprimindo colorido as transformaes */
      for (j = n; j >= 0; j--)
	if (j == i) {
	  j -= w - 1;
	  for (num = i; num > (i - w); num--) printf ("\e[1;34m%2d\e[0m ", wmof[num]);
	}
	else printf ("%2d ", wmof[j]);  
      printf ("\n");
      i -= w;    
    }
  }
  printf ("w = %d --> %d contadores\n", w, cont);

}/* wMOF ----------------------------------------------------------------------*/


/******************************* calcula_wMOF *********************************/
/* Funo que chama a funo que calcula o wMOF de um string binrio e        */
/* imprime o resultado.                                                       */
/* Recebe:                                                                    */
/*      - um inteiro n --> o nmero de iteraes                              */
/*      - um vetor de inteiros b --> a representao binria do nmero        */

void calcula_wMOF(int n, int w, int b[]) {
  int i, wmof[NMAX + 1];
  
  printf ("\n\n**************** wMOF *****************\n\n");
  
  wMOF(n, w, b, wmof);
  
  printf ("\nwMOF = ");
  for (i = n; i >= 0; i--)
    printf ("%2d ", wmof[i]);
  
  printf ("\n\n***************************************\n\n");
  
}/* calcula_wMOF -------------------------------------------------------------*/
