/******************************************************************************/
/* naf.c                                                                      */
/*                                                                            */
/* Contm as funes de clculo do NAF e wNAF.                                */
/******************************************************************************/

#include "funcoes.h"
#include "mof.h"
#include "naf.h"


/************************************ NAF *************************************/
/* Funo que calcula o NAF de um string binrio                              */
/* Recebe:                                                                    */
/*      - um inteiro n --> o nmero de iteraes                              */
/*      - um vetor de inteiros b --> a representao binria do nmero        */
/*      - um vetor de inteiros naf, onde vai ser "devolvida" a forma NAF de d */

void NAF (int n, int b[], int naf[]) {
  int i, j;
  
  for(i = 0; i < n ; i++) 
    naf[i] = b[i];
  naf[n] = 0;
  
  i = 0;
  while ((i + 2) <= n) {
    
    if (naf[i] == 1 && naf[i + 1] == 1) {
      
      /* imprimindo colorido */
      for (j = n; j >= 0; j--) {
	if (naf[j] == 1 && naf[j - 1] == 1)  {
	  printf ("\e[1;34m%2d\e[0m ", naf[j]);
	  printf ("\e[1;34m%2d\e[0m ", naf[--j]);
	}
	else printf ("%2d ", naf[j]);
      }
      printf ("\n");
      
      /* transformao NAF */
      naf[i] = -1;
      naf[i + 1] = 0;
      naf[i + 2] = 1;

      /* imprimindo colorida a transformao */
      for (j = n; j >= 0; j--) {
	if (naf[j] == 1 && naf[j - 2] == -1)  {
	  printf ("\e[1;34m%2d\e[0m ", naf[j]);
	  printf ("\e[1;34m%2d\e[0m ", naf[--j]);
	  printf ("\e[1;34m%2d\e[0m ", naf[--j]);
	}
	else printf ("%2d ", naf[j]);
      }
      printf ("\n");

      i += 2;
    }
    else i++;
  }
  
}/* NAF ----------------------------------------------------------------------*/


/******************************** calcula_NAF *********************************/
/* Funo que chama a funo que calcula o NAF de um string binrio e         */
/* imprime o resultado.                                                       */
/* Recebe:                                                                    */
/*      - um inteiro n --> o nmero de iteraes                              */
/*      - um vetor de inteiros b --> a representao binria do nmero        */

void calcula_NAF(int n, int b[]) {
  int i, naf[NMAX + 1];
  
  printf ("\n\n***************** NAF *****************\n\n");
  
  NAF(n, b, naf);
  
  printf ("\nNAF = ");
  for (i = n; i >= 0; i--)
    printf ("%2d ", naf[i]);
  
  printf ("\n\n***************************************\n\n\n");
  
}/* calcula_NAF --------------------------------------------------------------*/


/*********************************** wNAF *************************************/
/* Funo que calcula o wNAF de um string binrio                             */
/* Recebe:                                                                    */
/*    - um inteiro n --> o nmero de iteraes                                */
/*    - um inteiro w --> a largura da janela usada pelo wNAF                  */
/*    - um vetor de inteiros b --> a representao binria do nmero          */
/*    - um vetor de inteiros wnaf, onde vai ser "devolvida" a forma wNAF de d */

void wNAF (int n, int w, int b[], int wnaf[]) {
  int i, j, num, potencia, aux[NMAX + 1];
  int cont = 1, k;

  /* Caso w == 1 */
  if (w == 1) {
    for (i = 0; i < n; i++) {
      wnaf[i] = b[i]; 
    }
    wnaf[n] = 0; 
  }

  else {
  
    /* Transformando para a forma MOF */
    MOF(n, b, aux);
    
    i = 0;
    while ((w + i) < n) {
      num = 0;
      potencia = 1;

      if (aux[i] != 0) {

	/* Transformando para representao decimal */
	for (j = i; j < (w + i); j++) {
	  num += aux[j] * potencia;
	  potencia *= 2;
	}

	/* Preenchendo o vetor wnaf com o valor encontrado */
	potencia = pot (2, w);
	if (num < pot (2 , (w - 1))) wnaf[i] = num;
	else wnaf[i] = num - potencia;
	i += w;
      }

      /* Se o bit na forma MOF for 0, v para o prximo bit */
      else {
	wnaf[i] = 0;
	i++;
      }
      cont++;

      /* Imprimindo passo a passo, sem cor */
      for (j = n; j >= 0; j--)
	printf ("%2d ", wnaf[j]);
      printf ("\n"); 
    }
  }

  printf ("w = %d --> %d contadores\n", w, cont);

}/* wNAF ---------------------------------------------------------------------*/


/******************************* calcula_wNAF *********************************/
/* Funo que chama a funo que calcula o wNAF de um string binrio e        */
/* imprime o resultado.                                                       */
/* Recebe:                                                                    */
/*      - um inteiro n --> o nmero de iteraes                              */
/*      - um inteiro w --> a largura da janela usada pelo wNAF                */
/*      - um vetor de inteiros b --> a representao binria do nmero        */

void calcula_wNAF(int n, int w, int b[]) {
  int i, wnaf[NMAX + 1];
  
  printf ("\n\n**************** wNAF *****************\n\n");
  
  wNAF (n, w, b, wnaf);
  
  printf ("\nwNAF = ");
  for (i = n; i >= 0; i--)
    printf ("%2d ", wnaf[i]);
  
  printf ("\n\n***************************************\n\n");

}/* calcula_wNAF -------------------------------------------------------------*/


/********************************* wNAF_Int ***********************************/
/* Funo que calcula o wNAF a partir de um nmero inteiro d de n bits        */
/* Recebe:                                                                    */
/*    - um inteiro n --> o nmero de iteraes                                */
/*    - um inteiro nmero --> o nmero dado pelo usurio                      */
/*    - um inteiro w --> a largura da janela usada pelo wNAF                  */
/*    - um vetor de inteiros wnaf, onde vai ser "devolvida" a forma wNAF de d */

void wNAF_Int (int n, int numero, int w, int wnaf[]) {
  int aux, potencia, i = 0;
  int cont = 1;

  /* Caso w == 1 */
  if (w == 1) {
    aux = converte(numero, wnaf);
    wnaf[aux] = 0;
    cont = aux;
  }

  else {
  
    potencia = pot (2, w);
    while (numero > 0) {

      if (numero % 2 == 0) {
	wnaf[i] = 0;
      }

      else {
	aux = numero % potencia;           
	if (aux < pot (2 , (w - 1))) wnaf[i] = aux;
	else wnaf[i] = aux - potencia;
	numero -= wnaf[i];
      }
      cont++;
      numero /= 2;

      /* Imprimindo colorido as transformaes */
      for (aux = n; aux >= 0; aux--)
	 if(aux != i) printf ("%2d ", wnaf[aux]);
	 else printf ("\e[1;34m%2d\e[0m ", wnaf[aux]);
      printf ("\n");

      i += 1;
    }
  }
  printf ("w = %d --> %d contadores\n", w, cont);
  
}/* wNAF_Int -----------------------------------------------------------------*/


/***************************** calcula_wNAF_Int *******************************/
/* Funo que chama a funo que calcula o wNAF de um nmero inteiro d e      */
/* imprime o resultado.                                                       */
/* Recebe:                                                                    */
/*      - um inteiro n --> o nmero de iteraes                              */
/*      - um inteiro nmero --> o nmero dado pelo usurio                    */
/*      - um inteiro w --> a largura da janela usada pelo wNAF                */

void calcula_wNAF_Int(int n, int numero, int w) {
  int i, wnafInt[NMAX + 1];

  printf ("\n\n*************** wNAF_Int **************\n\n");
  
  wNAF_Int (n, numero, w, wnafInt);

  printf ("\nwNAF = ");
  for (i = n; i >= 0; i--)
    printf ("%2d ", wnafInt[i]);
  
  printf ("\n\n***************************************\n\n");

}/* calcula_wNAF_Int ---------------------------------------------------------*/
